import sys
import abstra.dashes as abstra_dashes
from .hf import AuthResponse


def overload_stdio(broker):
    def writeWraper(type, write, text):
        try:
            write(text)
            broker.send({"type": type, "payload": text})
        finally:
            return len(text)

    stdout_write = sys.stdout.write
    stderr_write = sys.stderr.write

    sys.stdout.write = lambda text: writeWraper("stdout", stdout_write, text)
    sys.stderr.write = lambda text: writeWraper("stderr", stderr_write, text)


def overload_abstra_sdk(broker):
    def get_user():
        broker.send({"type": "auth:initialize"})
        while True:
            type, data = broker.recv()
            if type == "auth:validation-ended":
                return AuthResponse(data["email"])

    abstra_dashes.get_user = get_user



import base64, json


def serialize(obj):
    return json.dumps(obj)


def deserialize(st):
    return json.loads(st)


def is_serializable(st):
    try:
        serialize(st)
        return True
    except:
        return False


def btos(b64):
    return base64.b64decode(b64).decode()


def get_staticmethod(cls, name):
    # hack because python sucks
    method_key = f"_{cls.__name__}{name}"
    return getattr(cls, method_key, None)


def convert_answer(cls, value):
    convert = get_staticmethod(cls, "__convert_answer")
    return convert(value) if convert else value


def revert_value(cls, value):
    revert = get_staticmethod(cls, "__revert_value")
    return revert(value) if revert else value


def read_file(filename: str):
    with open(filename, "r") as f:
        return f.read()



import pkg_resources


def version():
    return pkg_resources.get_distribution("abstra-runtimes").version


# import requests

# def check_version():
#     try:
#         __version__ = version()
#         libs = requests.get(
#             "https://hackerforms-api.abstra.cloud/public/abstra-pypi-packages"
#         ).json()
#         hackerforms = list(filter(lambda lib: lib["name"] == "hackerforms", libs))[0]
#         if hackerforms["version"] != __version__ and __version__ != "0.0.0":
#             print("You are using an outdated version of hackerforms.")
#             print(
#                 "Please update your library using the following command: pip install hackerforms=="
#                 + hackerforms["version"]
#             )
#     except Exception as e:
#         pass


if __name__ == "__main__":
    print(version())



import os

# this stops hackerforms from opening a browser
original_runner = os.getenv("RUNNER")
os.environ["RUNNER"] = "dash"
from hackerforms.widget_utils import get_widget_class  # exported
from hackerforms.auth import AuthResponse  # exported

if original_runner:
    os.environ["RUNNER"] = original_runner
